import base64
import datetime
import tempfile
import os
from ..gcs.add_file import add_file_to_gcs, get_image_file_name
from ..logging import log


def upload_doc_images(metadata):
    image_base64 = metadata.get('image_base64')
    # upload an image to the objectId/img folder
    if image_base64 and len(image_base64) > 100:
        image_data = base64.b64decode(image_base64)

        # Determine the file extension based on the MIME type
        mime_type = metadata.get("image_mime_type", "")
        object_id = metadata.get("objectId", "image")
        log.info(f"Found image_base64 for {object_id}")
        timestamp = datetime.datetime.now().strftime("%Y%m%d%H%M%S")
        image_path = get_image_file_name(object_id, image_name=timestamp, mime_type=mime_type)
        
        # Write image data to a temporary file
        with tempfile.NamedTemporaryFile(mode='wb', delete=False) as temp_image:
            temp_image.write(image_data)
            temp_image.flush()  # Make sure all data is written to the file

        temp_image_path = temp_image.name

        # wipe this so it doesn't get stuck in loop
        metadata["image_base64"] = None
        metadata["uploaded_to_bucket"] = True
        
        # Use the provided function to upload the file to GCS
        image_gsurl = add_file_to_gcs(
            filename=temp_image_path,
            vector_name=metadata["vector_name"],
            bucket_name=metadata["bucket_name"],
            metadata=metadata,
            bucket_filepath=image_path
        )
        os.remove(temp_image.name)
        log.info(f"Uploaded image to GCS: {image_gsurl}")

        return image_gsurl




import os
import socketio

from fastapi import FastAPI, Request
from fastapi.responses import HTMLResponse

from sunholo.logging import log



from fastapi.middleware.cors import CORSMiddleware


def create_fastapi_app():
    """Creates and configures a FastAPI app for image-based Q&A with Socket.IO integration.

    Args:
        image_qna_stream_fn: Streaming Q&A function (likely from gemini.genai)
        image_qna_fn: Non-streaming Q&A function (likely from gemini.genai)

    Returns:
        FastAPI: The configured FastAPI app instance.
    """

    # Create Socket.IO server and FastAPI app 
    sio = socketio.AsyncServer(async_mode='asgi')
    app = FastAPI()
    socket_app = socketio.ASGIApp(sio, other_asgi_app=app)
    app.mount("/ws/socket.io/", socket_app)

    # CORS Configuration 
    app.add_middleware(
        CORSMiddleware,
        allow_origins=["*"],
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )

    # Handle Socket.IO events, e.g., a connection
    @sio.event
    async def connect(sid, environ):
        log.info("Socket.IO client connected", sid)

    # Homepage Route
    @app.get("/", response_class=HTMLResponse)
    async def homepage(request: Request):
        return """
        <!DOCTYPE html>
        <html>
        <head>
            <title>Multivac Service Homepage</title>
        </head>
        <body>
            <h1>Welcome to Multivac</h1>
            <p>This is a debug homepage to confirm the service is up and running.</p>
        </body>
        </html>
        """

    return app 


#   Copyright [2024] [Holosun ApS]
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
from ..logging import log



from ..utils import load_config_key, load_config

def route_qna(vector_name):

    agent_type = load_config_key('agent', vector_name, filename='config/llm_config.yaml')
    log.info(f'agent_type: {agent_type}')

    agent_route, _ = load_config('config/cloud_run_urls.json')
    log.info(f'agent_route: {agent_route}')

    try:
        agent_url = agent_route[agent_type]
    except KeyError:
        raise ValueError(f'agent_url not found for {agent_type}')
    
    log.info(f'agent_url: {agent_url}')
    return agent_url

def route_endpoint(vector_name):

    agent_type = load_config_key('agent', vector_name, filename='config/llm_config.yaml')

    stem = route_qna(vector_name)

    agent_config, _ = load_config('config/agent_config.yaml')

    # Select the appropriate configuration based on agent_type
    if agent_type in agent_config:
        endpoints_config = agent_config[agent_type]
    else:
        endpoints_config = agent_config['default']

    # Replace placeholders in the config
    endpoints = {}
    for key, value in endpoints_config.items():
        format_args = {'stem': stem}
        if '{vector_name}' in value and vector_name is not None:
            format_args['vector_name'] = vector_name
        endpoints[key] = value.format(**format_args)

    return endpoints




from setuptools import setup, find_packages

# Define your base version
version = '0.40.5'

setup(
    name='sunholo',
    version=version,
    packages=find_packages(),
    license='Apache License, Version 2.0',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    description='Large Language Model DevOps - a package to help deploy LLMs to the Cloud.',
    author = 'Holosun ApS',
    author_email = 'multivac@sunholo.com',
    url = 'https://github.com/sunholo-data/sunholo-py',
    download_url=f'https://github.com/sunholo-data/sunholo-py/archive/refs/tags/v{version}.tar.gz',
    keywords=['llms', 'devops','google_cloud_platform'],
    package_data={
        'sunholo.database': ['sql/sb/*.sql'],
        'sunholo.lookup': ['*.yaml']
    },
    install_requires=[
        # List your dependencies here
        "google-cloud-alloydb-connector[pg8000]",
        "google-cloud-logging",
        "google-cloud-storage",
        "lancedb",
        "langchain",
        "langchain_experimental",
        "langchain-community",
        "langchain-openai",
        "langchain-google-genai",
        "langchain-anthropic",
        "pg8000",
        "sqlalchemy"
        # pydantic==1.10.13
        # "supabase",
        # "openai",
        # "tiktoken",
        # "google-cloud-storage",
        # "google-api-python-client",
        # "google-auth-httplib2",
        # "google-auth-oauthlib",
        # "psycopg2-binary"
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',      # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: Apache Software License', 
        'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12'
    ],
)
